#********************************************************************************************************************
#
#                          SOURCE Script to Process Input Data for Plotting Scripts
#
#                             Run as Nested SOURCE Script Called in Master_Script
#                             
#               In addition to the Model and Measure dataframs, this script creates two nested dataframes that 
#                    extract ONLY those segments/Pcodes that exceed the Min_Obs or Min_Obs_BoxDens cutoffs
#                         
#                    NestedData_Box used by the Boxplot, Density Plot, and Geomean Plotting functions
#                         
#                         NestedData_Multi used by the Multipanel Plotting and Plotly functions
#                                                      
#
#********************************************************************************************************************


#####################
# Error message test for grouping calibration stations by WASP segment
#Measured_Paired <- Measured_Paired %>%
#  dplyr::mutate(Segment_ID = gsub("100", 79, Segment_ID)) %>%
#  dplyr::mutate(Segment_ID = gsub("54|169", 39, Segment_ID))
####################    

#********************************************************************************************************************
#                                  QA that PCodes and Segments were Selected to Calibrate
#********************************************************************************************************************

#############################
# Error message test for blank Pcode and Station Lookup
# PCode_Lookup <- PCode_Lookup %>%
#  dplyr::filter(Calibrate != "X")
# 
# Station_Order_Lookup <- Station_Order_Lookup %>%
#  dplyr::filter(is.na(Segment_ID))
################################

if(nrow(PCode_Lookup) == 0 | nrow(Station_Order_Lookup) == 0){
  stop("No PCodes and/or WASP segments were selected for calibration in the lookup table")
} else {
  
  #********************************************************************************************************************
  #                                                 Segment/Station Order Lookup
  #********************************************************************************************************************
  Site_Order <- unique(fct_inorder(Station_Order_Lookup$Plot_Title))                                                          ## Sets order of sites based on calibration data order  
  Segment_Order <- unique(fct_inorder(as.factor(Station_Order_Lookup$Segment_ID)))                                          ## Sets order of WASP Segments based on WASP Segment ID
  
  
  
  #********************************************************************************************************************
  #                                                 Segment/Station Order Lookup
  #********************************************************************************************************************
 # Seg_Order <- unique(fct_inorder(Station_Order_Lookup$Segment_ID))
  
  
  #********************************************************************************************************************
  #                                 QA: Are the WASP Segments in the Model Data Selected in the Lookup Table
  #********************************************************************************************************************
  
  
  if(length(setdiff(unique(Model_Intrp$Segment_ID), unique(Station_Order_Lookup$Segment_ID))) != 0) {
    warning(paste("The Processed WASP datafile had records for WASP Segment(s) ", paste(setdiff(unique(Model_Intrp$Segment_ID), unique(Station_Order_Lookup$Segment_ID)), collapse = ", "), 
                  ", but the segment(s) was not selected for calibration in the lookup table \nThe WASP segment(s) were excluded from calibration.", sep = ""))
  }
  
  
  #********************************************************************************************************************
  #                                       Adds Missing Columns
  #********************************************************************************************************************
  Measured_Paired <- Measured_Paired %>%
    #dplyr::mutate(Date_Time = ymd_hms(Date_Time, tz = Model_TZ, truncated = 3)) %>%                                               ## Formats the date as POSIX
    dplyr::mutate(Year = year(Date_Time),                                                                                             ## Adds a Year column
           Month = lubridate::month(Date_Time)) %>%                                                                                       ## Adds a Month columns
    dplyr::mutate(RCode = toupper(RCode)) %>%                                                                                         ## Changes all RCodes to uppercase                                                                                    
    dplyr::mutate(Station_ID_Mrgd = Station_Order_Lookup$Station_ID_Mrgd[match(Segment_ID, Station_Order_Lookup$Segment_ID)],     ## Adds the Merged Station Name Column, set as factor
           Plot_Title = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title)),                                  ## Adds the Station Name Column, set as factor
           Segment_ID = factor(Segment_ID, levels = unique(Station_Order_Lookup$Segment_ID))) %>%                        ## Allows Segment_ID order to be set
    drop_na(Plot_Title)                                                                                                        ## If Input Data has more Segments/Calib Stats than Lookup, it causes missing plot title. Drops WASP/Calib segments that can't be mapped to plot title.
  
  Model_Intrp <- Model_Intrp %>%
    #dplyr::mutate(Date_Time = ymd_hms(Date_Time, tz = Model_TZ, truncated = 3)) %>%                                               ## Formats the date as POSIX, records missing h:m:s set to midnightS
    dplyr::mutate(Segment_ID = factor(Segment_ID, levels = unique(Station_Order_Lookup$Segment_ID))) %>%
    dplyr::mutate(Year = year(Date_Time),                                                                                             ## Adds a Year column
           Month = lubridate::month(Date_Time)) %>%                                                                                       ## Adds a Month column
    gather(-Date_Time, -Station_ID, -Plot_Title, -Year, -Month, -Segment_ID, 
           key = "PCode", value = "Result_Sim", na.rm = TRUE) %>%                                                              ## Reshapes the DF into long format
    dplyr::rename(Result_Sim_Full = Result_Sim) %>%
    dplyr::mutate(Station_ID_Mrgd = Station_Order_Lookup$Station_ID_Mrgd[match(Segment_ID, Station_Order_Lookup$Segment_ID)],     ## Adds the Merged Station Name Column, set as factor
           Plot_Title = factor(Plot_Title, levels = unique(Station_Order_Lookup$Plot_Title))) %>%
    drop_na(Plot_Title)                                                                                                        ## If Input Data has more Segments/Calib Stats than Lookup, it causes missing plot title. Drops WASP segments/Calib Segments that can't be mapped to plot title.
  
  Measured_Paired <- Measured_Paired %>%
    dplyr::filter(Date_Time >= min(Model_Intrp$Date_Time) & Date_Time <= max(Model_Intrp$Date_Time))                                      ## Subsets the measured data to only the model period
  
  
  
  #********************************************************************************************************************
  #                                 QA All Selected Calibration Segments in Model Output File
  #********************************************************************************************************************
  
  if(length(setdiff(Segment_Order, unique(Model_Intrp$Segment_ID))) != 0) {
    warning(paste("WASP Segment(s)", paste(setdiff(Segment_Order, unique(Model_Intrp$Segment_ID)), collapse = ", "), 
                  "were selected for calibration in Lookup Table, but had no records in the Model Data. \nThe missing WASP segment was not included in calibration", sep = " "))
    Station_Order_Lookup <- Station_Order_Lookup %>%
      dplyr::filter(Segment_ID %in% unique(Model_Intrp$Segment_ID))
  }
  
  
  
  #********************************************************************************************************************
  #                                        QA to find missing records and pcodes in model data
  #********************************************************************************************************************
  
  ################ Error message test for missing record
  #      Model_Intrp <- Model_Intrp %>%
  #        dplyr::mutate(Result_Sim_Full = case_when(PCode == "TEMP" & Segment_ID == "39" ~ as.numeric(NA),
  #                                           PCode == "TN" & Segment_ID == "46" ~ as.numeric(NA),
  #                                           TRUE ~ Result_Sim_Full)) %>%
  #        dplyr::filter(!is.na(Result_Sim_Full))
  ###################   
  
  Mod_Invry <- Model_Intrp %>%
    group_by(PCode, Segment_ID, Station_ID, Plot_Title) %>%
    summarize(Numb_ModelObs = n()) %>%
    as.data.frame() %>%
    dplyr::mutate(Station_ID_Mrgd = Station_Order_Lookup$Station_ID_Mrgd[match(Segment_ID, Station_Order_Lookup$Segment_ID)]) %>%
    select(Segment_ID, Station_ID, Plot_Title, Station_ID_Mrgd, PCode, Numb_ModelObs)
  
  QA_Summary <- Measured_Paired %>%                                                                                         ## Reports inventory of measured data by station and PCode      
    group_by(PCode, Segment_ID, Station_ID, Plot_Title) %>%
    summarize(Numb_MeasObs = n()) %>%
    as.data.frame() %>%
    arrange(Segment_ID, Plot_Title, PCode) %>%
    dplyr::mutate(Station_ID_Mrgd = Station_Order_Lookup$Station_ID_Mrgd[match(Segment_ID, Station_Order_Lookup$Segment_ID)]) %>%
    select(Segment_ID, Station_ID_Mrgd, Station_ID, Plot_Title, PCode, Numb_MeasObs) %>%
    left_join(Mod_Invry, by = c("Segment_ID", "Station_ID", "Plot_Title", "Station_ID_Mrgd", "PCode")) 
  
  # Writeout QA table to find minimum number of records *************************
  if(nrow(QA_Summary) != 0) {                                                                                                         ## Writes out PCode by Station summary table
    wb_QA <-createWorkbook() 
    
    addWorksheet(wb_QA, sheetName = "Numb_Rec_by_Station")                                                                            ## Adds worksheet for record count
    
    addWorksheet(wb_QA, sheetName = "WASP_Station_ID_Crswlk")                                                                         ## Adds worksheet for crosswalk
    
    writeData(wb_QA, sheet = "Numb_Rec_by_Station", QA_Summary)                                                                       ## Adds data to record count worksheet
    
    writeData(wb_QA, sheet = "WASP_Station_ID_Crswlk", Station_Order_Lookup)                                                          ## Adds data to crosswalk worksheet
    
    setColWidths(wb_QA, sheet = "Numb_Rec_by_Station", cols = c(1:ncol(QA_Summary)), widths = "auto")                                 ## Sets widths of columns
    
    setColWidths(wb_QA, sheet = "WASP_Station_ID_Crswlk", cols = c(1:ncol(Station_Order_Lookup)), widths = "auto")                    ## Sets widths of columns
    
    openxlsx::saveWorkbook(wb_QA, paste(str_remove(Path_Data_Out, "/Proc_Out"), "/Stats/QA_Smry_", Model_Run_Code, ".xlsx", sep = ""), overwrite = TRUE)       ## Saves the finished workbook
    
    
  } else {                                                                                                                            ## Writes out a placeholder table if Crosswalk table is empty
    write.xlsx(data.frame(QA_Record_Summary = "Measured dataframe contains no data records"), 
               file = paste(str_remove(Path_Data_Out, "/Proc_Out"), "/Stats/QA_Smry_", Model_Run_Code, ".xlsx", sep = ""), sheetName = "Numb_Rec_by_Station", 
               append = FALSE, showNA = FALSE, col.names = TRUE, row.names = FALSE )
  }
  
  Missing <- bind_rows(Missing_Model = setdiff(distinct(Measured_Paired, Station_ID, Plot_Title, Segment_ID, PCode),                   ## Creates DF of records missing from both DFs
                                               distinct(Model_Intrp, Station_ID, Plot_Title, Segment_ID, PCode)), 
                       Missing_Meas = setdiff(distinct(Model_Intrp,Station_ID, Plot_Title, Segment_ID, PCode), 
                                              distinct(Measured_Paired, Station_ID, Plot_Title, Segment_ID, PCode)), 
                       .id = "Where_Missing") %>%
    dplyr::filter(., Where_Missing == "Missing_Model") 
  
  ########### Error message test for missing records
  # Add <- data.frame(Where_Missing = "Missing_Model", Station_ID = c("Station_A", "Station_B"), Plot_Title = c("Full_A", "FULL_B"), Segment_ID = c(43, 88), PCode = c("TN", "TP")   )
  #  Missing <- rbind(Missing, Add)
  ####################### 
  
  if(count(Missing) != 0) {                                                                                                            ## Error / ends script if Model DF is missing a Station/PCode combination
    stop(paste("Input model data is missing a PCode/Station combination.
             \n Examine input model data and QA_Summary for stations--", 
               paste(unique(paste("WASP Segment", Missing$Segment_ID, "/", Missing$Plot_Title, sep ="")), collapse = "; ")))         ## Error message reporting missing records
  }
  
  #********************************************************************************************************************
  #                         Preprocesses the Measured and Modeled DF into a single nested DF
  #                         
  #                 This creates a nested DF that excludes any segment that did not meet the Min_Obs_BoxDens cutoff and 
  #                           will be used by the plotting custom functions and walk2 to generate figures
  #                 
  #********************************************************************************************************************
  
  #############
  ##ERROR Check to test Min_Obs_Cutoff
  ##Min_Obs_BoxDens <- 300
  ##############

  
  NestData_Box <- Measured_Paired %>%                                                                                                                                                   ## Uses Measured Paired as Starting Point
    dplyr::mutate(RCode = Rcode_Mapping$RCode_Flag[match(RCode, Rcode_Mapping$RCode)])  %>%                                                                                                ## Maps Rcode to Rcode Flag
    dplyr::mutate(Units = str_replace(Units, "^u", "\u03BC")) %>%                                                                                                                       ## Creates HTML code to display micro symbol
    group_nest(PCode, .key = "Meas_Data", keep = TRUE) %>%                                                                                                                              ## Group nests based on Pcode
    left_join(., Model_Intrp %>% group_nest(PCode, .key = "Model_Data", keep = TRUE), by = c("PCode"))  %>%                                                                             ## Uses left_join to pair Meas and Model data (will drop Pcodes that did not have measured records AT ANY STATION)
    mutate(Meas_Filt = map(.x = Meas_Data, ~ group_by(.x, Segment_ID) %>% dplyr::filter(., n() >= Min_Obs_BoxDens))) %>%                                                              ## Creates filtered nested DF that excludes Segment_IDs that did not have enough records
    mutate(MeasSeg_Box = map(.x = Meas_Filt, ~unique(.$Segment_ID))) %>%                                                                                                              ## Creates nested list of WASP Segments that had records for that PCOde
    {if(Min_Obs_BoxDens != 0) mutate(., Model_Filt = map2(.x = Model_Data, .y = MeasSeg_Box, ~dplyr::filter(., .$Segment_ID %in% .y))) else mutate(., Model_Filt = Model_Data)} %>%   ## When Min_Obs_BoxDens > 0, it drops any WASP segment that did not have enough measured records, otherwise, uses full model data 
    select(-Model_Data, -Meas_Data)                                                                                                                                                     ## Drops original data to save filesize
  
  #********************************************************************************************************************
  #                          Preprocessed Nested DF into a new DF that is grouped by Segment and Pcode
  #                
  #                 This creates a nested DF that excludes any segment that did not meet the Min_Obs cutoff and 
  #                           will be used by the multipanel FUNCTION and Plotly FUNCTIONS
  #********************************************************************************************************************
  
  #############
  ##ERROR Check to test Min_Obs_Cutoff
  ##Min_Obs <- 300
  ##############
  
  
   NestData_Multi <- Measured_Paired %>%                                                                                                                                                 ## Uses Measured Paired as Starting Point
    dplyr::mutate(RCode = Rcode_Mapping$RCode_Flag[match(RCode, Rcode_Mapping$RCode)])  %>%                                                                                                 ## Maps Rcode to Rcode Flag
     dplyr::mutate(Units = str_replace(Units, "^u", "\u03BC")) %>%                                                                                                                       ## Creates HTML code to display micro symbol
     group_by(Segment_ID, PCode) %>%                                                                                                                                                   ## Groups DF by Segment and PCode
     dplyr::filter(n() >= Min_Obs) %>%                                                                                                                                                   ## Drops any segment that does not have sufficient records
     ungroup() %>%                                                                                                                                                                       ## Ungroups DF
     group_nest(Segment_ID, PCode, .key = "Meas_Filt", keep = TRUE) %>%                                                                                                                ## Group nests DF by PCode and Segment
     left_join(., Model_Intrp %>% group_nest(Segment_ID, PCode, .key = "Model_Filt", keep = TRUE), by = c("Segment_ID", "PCode")) %>%                                                   ## Left joins to the Model_Intrp DF that is similarly nested
     arrange(factor(PCode, levels = fct_inorder(as.factor(PCode_Lookup$PCode))), Segment_ID) %>%
     rowid_to_column(var = "Fig_Numb")                                                                                                                                                  ## Adds Row ID to allow for Figure Numbering argument in MultiCalib
  
  }



