
### NEXT STEP IS TO FINALIZE THE FORMATTING/PREPROCESSING TO MATCH THE FORMAT OF THE ACTUAL INPUT/OUTPUT
### ADD A PLOTTING FUNCTION
### ALSO, DETERMINE IF I WANT TO INCORPORATE DATARETRIEVAL DIRRECTLY INTO THESE SCRIPTS, OR STILL USE 
### THE NWIS DOWNLOAD SCRIPTS TO DOWNLOAD THE USGS FLOW/GAGE HEIGHT DATA THAT ARE USED HERE

#********************************************************************************************************************
#              Proof of concept script that will read in a SONDE datafile and a NWIS flow/gage height datafile and
#              pair the data when the timestep of the SONDE data do not match the time step of the flow/gage data
#           
#         Based on the timestamp of the SONDE data, will extract the flow/gage height record for the closest flow measurement
#         
#         This can account for SONDE data being reported every 30-min, while flow/gage height are measured every 15-min
#         
#                               Will output the paired data as a new dataframe
#                               
#                               
#               ****NOTE: This script does not linearly interpolate the gage/flow data, it simply reports the actual measured flow/gage height 
#                with a dateTime that is closest to the HOBO temperature timestamp. 
#             
#             
#       The scripts also currently assume the HOBO data are in the form of a NWIS SONDE dataRetrieval pull, and that the USGS gage height data were similarly generated using dataRetrieval. 
#        The input data will need to be reformatted based on the actual HOBO dataframe format and the final output
#
#********************************************************************************************************************

## THESE NEXT LINES CAN BE DELETED BC THEY WERE USED TO CREATE AN EXAMPLE HOBO DATAFRAME
 #WQ_Temp_Shifted <- Keep_Merged_WQ_Raw %>%
 #renameNWISColumns(p00011 = "WTEMPF") %>%
 #dplyr::filter(minute(dateTime) %in% c(0,30)) %>%
 #rowwise(.) %>%
 #mutate(dateTime = dateTime + minutes(sample(seq(1:5), 1))) %>%
 #as.data.frame(.)


 #write.table(WQ_Temp_Shifted,                                                                                               ## Writes out .csv listing query settings
#            file = paste("./Mrg_Test/Datashifted_WQ_TEST.csv", sep = ""), 
#            row.names = FALSE, col.names = TRUE, sep = ",")

## END OF DELETE THIS SECTION

#********************************************************************************************************************
#     Sets time zone for the input data, assumes the timestamp for both the temparature and flow data are already in this timezone
#     
#                            This allows the data to be correctly paired      
#
#********************************************************************************************************************

## Acceptable time zones US/Eastern, US/Central, US/Mountain, US/Pacific, UTC

TimeZone <- "US/Eastern"                                                                       ## Sets time zone 

#********************************************************************************************************************
#             Sets filepaths and filenames for input data
#
#********************************************************************************************************************

Filepath_Mrg_Test <- "./Mrg_Test"                                                              ## Filepath where the input and output data are saved

Filename_NWIS_Raw <- "USGS_Instant_GH_TEST.csv"                                                ## Filename of the NWIS gage height data (or flow data)  

Filename_HOBO_Raw <- "Datashifted_WQ_TEST.csv"                                                 ## Filename of the temperature and WQ data, assumes data are in wide format

#********************************************************************************************************************
#             Reads in HOBO (water quality data) and NWIS flow/gage height data
#
#********************************************************************************************************************

HOBO_Raw <- read.csv(paste(Filepath_Mrg_Test, Filename_HOBO_Raw, sep = "/")) %>%
  mutate(dateTime = ymd_hms(dateTime, tz = TimeZone)) %>%                                      ## Converts to datetime
  rowid_to_column(var = "RowID_Source")                                                        ## Adds rowid for easier referencing

NWIS_Flow_Raw <- read.csv(paste(Filepath_Mrg_Test, Filename_NWIS_Raw, sep = "/")) %>%
  renameNWISColumns(.) %>%                                                                     ## Renames the gage height columns based on standard NWIS column names
  mutate(dateTime = ymd_hms(dateTime, tz = TimeZone)) %>%                                      ## Converts to datetime
  arrange(site_no, dateTime) %>%                                                               ## Sorts the dataframe by gage number and dateTime bc findInterval() needs dates sorted
  rowid_to_column(var = "RowID")

#********************************************************************************************************************
#             Creates nested dataframe based on HOBO and NWIS dataframes
#             
#       The nested dataframe allows the scripts to iterate through the data using Purrr
#       
#     This section will find the dateTime interval that the HOBO dateTime falls within, then will extract the NWIS dateTime for the record 
#     immediately before and after the HOBO timestamp and determine which of the two records is closer to the HOBO time stamp.  This closest
#     record will then be extracted based on row number and merged with the HOBO record (using cbind()) after appending all NWIS column names
#     with "_NWIS".
#     
#     HOBO_data = original HOBO WQ data
#     NWIS_data = original NWIS flow/gage height data
#     NWIS_Sub = NWIS flow/gage data that was filtered to the timestamp that is closest to the timestamp of the HOBO record
#     Pair_Data = HOBO WQ data with the merged NWIS Sub data that was filtered to the closest NWIS time stamp; this can be used to populate a QA file and fully processed merged data
#       
#     
#********************************************************************************************************************

Nest_Data <- HOBO_Raw %>%
  group_nest(site_no, .key = "HOBO_Data", keep = TRUE) %>%
  left_join(., NWIS_Flow_Raw %>% group_nest(site_no, .key = "NWIS_Data", keep = TRUE), by = "site_no") %>%
  ## This next line uses findInterval() to find the dateTime in the NWIS data that is immediately before and after the HOBO dateTime
  ## This rowindex is then referenced in subsequent lines of code to extract the NWIS records by row
  dplyr::mutate(HOBO_Data = map2(.x = HOBO_Data, .y = NWIS_Data, ~ mutate(.x, Pre_Index_NWIS = findInterval(.x$dateTime, .y$dateTime),
                                                                          Post_Index_NWIS = findInterval(.x$dateTime, .y$dateTime) + 1))) %>%
  ## When the closest dateTime is the last record in the NWIS dataframe, this next line uses the index of the pre record for both the pre and post NWIS dateTime
  dplyr::mutate(HOBO_Data = map2(.x = HOBO_Data, .y = NWIS_Data, ~ mutate(.x, Post_Index_NWIS = case_when(Post_Index_NWIS > nrow(.y) ~ Pre_Index_NWIS,
                                                                                                     TRUE ~ Post_Index_NWIS)))) %>%
  ## Calculates the time difference between the pre and post dateTime relative to the selected HOBO dateTime; uses the record that was closest to the HOBO dateTime (based on absolute values)
  ## If the pre and post are equally close to the selected dateTime, uses the Pre value by default
  dplyr::mutate(HOBO_Data = map2(.x = HOBO_Data, .y = NWIS_Data, ~ mutate(.x, Near_Index_NWIS = case_when(abs(.y$dateTime[Pre_Index_NWIS] - .x$dateTime) <= abs(.y$dateTime[Post_Index_NWIS] - .x$dateTime) ~ .x$Pre_Index_NWIS,
                                                                                                    TRUE ~ .x$Post_Index_NWIS)))) %>%
  ## Extracts the columns from the NWIS based on the closest index that was defined previously and appends all column names with "_NWIS" to identify those columns
  ## These filtered NWIS data are saved as a new nested dataframe
  dplyr::mutate(NWIS_Sub = map2(.x = HOBO_Data, .y = NWIS_Data, ~.y[.x$Near_Index_NWIS,] %>% select(-agency_cd, -tz_cd) %>% rename_with(~paste0(., "_NWIS")))) %>%
  ## Combines the filtered NWIS sub dataframe with the original HOBO dataframe and saved as a new Paired dataframe
  dplyr::mutate(Pair_Data = map2(.x = HOBO_Data, .y = NWIS_Sub, ~cbind(.x, .y))) %>%
  ## Calculates the absolute difference in the paired dateTimes, reported as minutes
  ## This can be used to find paired dateTime records with large differences
  dplyr::mutate(Pair_Data = map(.x = Pair_Data, ~mutate(.x, Time_Diff_mins = abs(difftime(dateTime, dateTime_NWIS, units = "mins")))))

#********************************************************************************************************************
#             Extracts the paired HOBO/NWIS dataframe and exports a .csv
#
#********************************************************************************************************************

Output <- Nest_Data %>%
  select(Pair_Data) %>%          ## selects and unnests the paired data
  unnest(Pair_Data)


write.table(Output, file = paste0(Filepath_Mrg_Test, "/Paired_WQ_NWIS_Proc_Out_", Sys.Date(), ".csv"), sep = ",", na = "", 
       col.names = TRUE, row.names = FALSE)






